@@ -2,15 +2,17 @@ use std::collections::{HashMap, HashSet};
 use std::ffi::OsString;
 use std::path::PathBuf;
 
+use bat::assets::HighlightingAssets;
 use clap::{AppSettings, ColorChoice, FromArgMatches, IntoApp, Parser};
 use lazy_static::lazy_static;
 use syntect::highlighting::Theme as SyntaxTheme;
 use syntect::parsing::SyntaxSet;
 
 use crate::config::delta_unreachable;
+use crate::env::DeltaEnv;
 use crate::git_config::{GitConfig, GitConfigEntry};
 use crate::options;
-use crate::utils::bat::assets::HighlightingAssets;
+use crate::utils;
 use crate::utils::bat::output::PagingMode;
 
 #[derive(Parser)]
@@ -337,7 +339,7 @@ pub struct Opt {
         default_value = "added:",
         value_name = "STRING"
     )]
-    /// Text to display in front of an added file path.
+    /// Text to display before an added file path.
     ///
     /// Used in the default value of navigate-regex.
     pub file_added_label: String,
@@ -347,7 +349,7 @@ pub struct Opt {
         default_value = "copied:",
         value_name = "STRING"
     )]
-    /// Text to display in front of a copied file path.
+    /// Text to display before a copied file path.
     pub file_copied_label: String,
 
     #[clap(
@@ -366,7 +368,7 @@ pub struct Opt {
         default_value = "",
         value_name = "STRING"
     )]
-    /// Text to display in front of a modified file path.
+    /// Text to display before a modified file path.
     ///
     /// Used in the default value of navigate-regex.
     pub file_modified_label: String,
@@ -376,7 +378,7 @@ pub struct Opt {
         default_value = "removed:",
         value_name = "STRING"
     )]
-    /// Text to display in front of a removed file path.
+    /// Text to display before a removed file path.
     ///
     /// Used in the default value of navigate-regex.
     pub file_removed_label: String,
@@ -386,7 +388,7 @@ pub struct Opt {
         default_value = "renamed:",
         value_name = "STRING"
     )]
-    /// Text to display in front of a renamed file path.
+    /// Text to display before a renamed file path.
     ///
     /// Used in the default value of navigate-regex.
     pub file_renamed_label: String,
@@ -489,7 +491,7 @@ pub struct Opt {
     pub hunk_header_style: String,
 
     #[clap(long = "hunk-label", default_value = "", value_name = "STRING")]
-    /// Text to display in front of a hunk header.
+    /// Text to display before a hunk header.
     ///
     /// Used in the default value of navigate-regex.
     pub hunk_label: String,
@@ -509,7 +511,7 @@ pub struct Opt {
     pub hyperlinks: bool,
 
     #[clap(long = "hyperlinks-commit-link-format", value_name = "FMT")]
-    /// Format string for commit hyperlinks (requiraes --hyperlinks).
+    /// Format string for commit hyperlinks (requires --hyperlinks).
     ///
     /// The placeholder "{commit}" will be replaced by the commit hash. For example:
     /// --hyperlinks-commit-link-format='https://mygitrepo/{commit}/'
@@ -1007,8 +1009,8 @@ pub struct Opt {
     #[clap(long = "wrap-max-lines", default_value = "2", value_name = "N")]
     /// How often a line should be wrapped if it does not fit.
     ///
-    /// Zero means to never wrap. Any content which does not fit will be truncated. A value of
-    /// "unlimited" means a line will be wrapped as many times as required.
+    /// Zero means to never wrap. Any content which does not fit after wrapping will be truncated. A
+    /// value of "unlimited" means a line will be wrapped as many times as required.
     pub wrap_max_lines: String,
 
     #[clap(
@@ -1029,7 +1031,7 @@ pub struct Opt {
     )]
     /// Pre-wrapped content symbol (right-aligned).
     ///
-    /// Symbol displayed in front of right-aligned wrapped content.
+    /// Symbol displayed before right-aligned wrapped content.
     pub wrap_right_prefix_symbol: String,
 
     #[clap(long = "wrap-right-symbol", default_value = "↴", value_name = "STRING")]
@@ -1071,6 +1073,9 @@ pub struct Opt {
 
     #[clap(skip)]
     pub git_config_entries: HashMap<String, GitConfigEntry>,
+
+    #[clap(skip)]
+    pub env: DeltaEnv,
 }
 
 #[derive(Default, Clone, Debug)]
@@ -1119,28 +1124,40 @@ impl Default for PagingMode {
 
 impl Opt {
     pub fn from_args_and_git_config(
+        env: DeltaEnv,
         git_config: Option<GitConfig>,
         assets: HighlightingAssets,
     ) -> Self {
-        Self::from_clap_and_git_config(Self::into_app().get_matches(), git_config, assets)
+        Self::from_clap_and_git_config(env, Self::into_app().get_matches(), git_config, assets)
     }
 
-    pub fn from_iter_and_git_config<I>(iter: I, git_config: Option<GitConfig>) -> Self
+    pub fn from_iter_and_git_config<I>(
+        env: DeltaEnv,
+        iter: I,
+        git_config: Option<GitConfig>,
+    ) -> Self
     where
         I: IntoIterator,
         I::Item: Into<OsString> + Clone,
     {
-        let assets = HighlightingAssets::new();
-        Self::from_clap_and_git_config(Self::into_app().get_matches_from(iter), git_config, assets)
+        let assets = utils::bat::assets::load_highlighting_assets();
+        Self::from_clap_and_git_config(
+            env,
+            Self::into_app().get_matches_from(iter),
+            git_config,
+            assets,
+        )
     }
 
     fn from_clap_and_git_config(
+        env: DeltaEnv,
         arg_matches: clap::ArgMatches,
         mut git_config: Option<GitConfig>,
         assets: HighlightingAssets,
     ) -> Self {
         let mut opt = Opt::from_arg_matches(&arg_matches)
             .unwrap_or_else(|_| delta_unreachable("Opt::from_arg_matches failed"));
+        opt.env = env;
         options::set::set_options(&mut opt, &mut git_config, &arg_matches, assets);
         opt.git_config = git_config;
         opt
