@@ -35,7 +35,7 @@ use std::process;
 use bytelines::ByteLinesReader;
 
 use crate::delta::delta;
-use crate::utils::bat::assets::{list_languages, HighlightingAssets};
+use crate::utils::bat::assets::list_languages;
 use crate::utils::bat::output::OutputType;
 
 pub fn fatal<T>(errmsg: T) -> !
@@ -65,6 +65,12 @@ pub mod errors {
 
 #[cfg(not(tarpaulin_include))]
 fn main() -> std::io::Result<()> {
+    // Do this first because both parsing all the input in `run_app()` and
+    // listing all processes takes about 50ms on Linux.
+    // It also improves the chance that the calling process is still around when
+    // input is piped into delta (e.g. `git show  --word-diff=color | delta`).
+    utils::process::start_determining_calling_process_in_thread();
+
     // Ignore ctrl-c (SIGINT) to avoid leaving an orphaned pager process.
     // See https://github.com/dandavison/delta/issues/681
     ctrlc::set_handler(|| {})
@@ -79,8 +85,13 @@ fn main() -> std::io::Result<()> {
 // report that two files differ when delta is called with two positional
 // arguments and without standard input; 2 is used to report a real problem.
 fn run_app() -> std::io::Result<i32> {
-    let assets = HighlightingAssets::new();
-    let opt = cli::Opt::from_args_and_git_config(git_config::GitConfig::try_create(), assets);
+    let assets = utils::bat::assets::load_highlighting_assets();
+    let env = env::DeltaEnv::init();
+    let opt = cli::Opt::from_args_and_git_config(
+        env.clone(),
+        git_config::GitConfig::try_create(&env),
+        assets,
+    );
 
     let subcommand_result = if opt.list_languages {
         Some(list_languages())
@@ -122,17 +133,24 @@ fn run_app() -> std::io::Result<i32> {
     }
 
     let mut output_type =
-        OutputType::from_mode(config.paging_mode, config.pager.clone(), &config).unwrap();
+        OutputType::from_mode(&env, config.paging_mode, config.pager.clone(), &config).unwrap();
     let mut writer = output_type.handle().unwrap();
 
-    if atty::is(atty::Stream::Stdin) {
-        let exit_code = subcommands::diff::diff(
-            config.minus_file.as_ref(),
-            config.plus_file.as_ref(),
-            &config,
-            &mut writer,
-        );
-        return Ok(exit_code);
+    match (config.minus_file.as_ref(), config.plus_file.as_ref()) {
+        (None, None) => {}
+        (Some(minus_file), Some(plus_file)) => {
+            let exit_code = subcommands::diff::diff(minus_file, plus_file, &config, &mut writer);
+            return Ok(exit_code);
+        }
+        _ => {
+            eprintln!(
+                "\
+    The main way to use delta is to configure it as the pager for git: \
+    see https://github.com/dandavison/delta#configuration. \
+    You can also use delta to diff two files: `delta file_A file_B`."
+            );
+            return Ok(config.error_exit_code);
+        }
     }
 
     if let Err(error) = delta(io::stdin().lock().byte_lines(), &mut writer, &config) {
