@@ -1,15 +1,19 @@
 use syntect::highlighting::Style as SyntectStyle;
 use unicode_segmentation::UnicodeSegmentation;
 
+use crate::cli;
 use crate::config::INLINE_SYMBOL_WIDTH_1;
+use crate::fatal;
 
 use crate::config::Config;
+use crate::delta::DiffType;
 use crate::delta::State;
 use crate::features::line_numbers::{self, SideBySideLineWidth};
 use crate::features::side_by_side::{available_line_width, line_is_too_long, Left, Right};
 use crate::minusplus::*;
 use crate::paint::LineSections;
 use crate::style::Style;
+use crate::utils::syntect::FromDeltaStyle;
 
 /// See [`wrap_line`] for documentation.
 #[derive(Clone, Debug)]
@@ -26,6 +30,92 @@ pub struct WrapConfig {
     pub inline_hint_syntect_style: SyntectStyle,
 }
 
+impl WrapConfig {
+    pub fn from_opt(opt: &cli::Opt, inline_hint_style: Style) -> Self {
+        Self {
+            left_symbol: ensure_display_width_1("wrap-left-symbol", opt.wrap_left_symbol.clone()),
+            right_symbol: ensure_display_width_1(
+                "wrap-right-symbol",
+                opt.wrap_right_symbol.clone(),
+            ),
+            right_prefix_symbol: ensure_display_width_1(
+                "wrap-right-prefix-symbol",
+                opt.wrap_right_prefix_symbol.clone(),
+            ),
+            use_wrap_right_permille: {
+                let arg = &opt.wrap_right_percent;
+                let percent = remove_percent_suffix(arg)
+                    .parse::<f64>()
+                    .unwrap_or_else(|err| {
+                        fatal(format!(
+                            "Could not parse wrap-right-percent argument {}: {}.",
+                            &arg, err
+                        ))
+                    });
+                if percent.is_finite() && percent > 0.0 && percent < 100.0 {
+                    (percent * 10.0).round() as usize
+                } else {
+                    fatal("Invalid value for wrap-right-percent, not between 0 and 100.")
+                }
+            },
+            max_lines: adapt_wrap_max_lines_argument(opt.wrap_max_lines.clone()),
+            inline_hint_syntect_style: SyntectStyle::from_delta_style(inline_hint_style),
+        }
+    }
+
+    // Compute value of `max_line_length` field in the main `Config` struct.
+    pub fn config_max_line_length(
+        &self,
+        max_line_length: usize,
+        available_terminal_width: usize,
+    ) -> usize {
+        match self.max_lines {
+            1 => max_line_length,
+            // Ensure there is enough text to wrap, either don't truncate the input at all (0)
+            // or ensure there is enough for the requested number of lines.
+            // The input can contain ANSI sequences, so round up a bit. This is enough for
+            // normal `git diff`, but might not be with ANSI heavy input.
+            0 => 0,
+            wrap_max_lines => {
+                let single_pane_width = available_terminal_width / 2;
+                let add_25_percent_or_term_width =
+                    |x| x + std::cmp::max((x * 250) / 1000, single_pane_width) as usize;
+                std::cmp::max(
+                    max_line_length,
+                    add_25_percent_or_term_width(single_pane_width * wrap_max_lines),
+                )
+            }
+        }
+    }
+}
+
+fn remove_percent_suffix(arg: &str) -> &str {
+    match &arg.strip_suffix('%') {
+        Some(s) => s,
+        None => arg,
+    }
+}
+
+fn ensure_display_width_1(what: &str, arg: String) -> String {
+    match arg.grapheme_indices(true).count() {
+        INLINE_SYMBOL_WIDTH_1 => arg,
+        width => fatal(format!(
+            "Invalid value for {}, display width of \"{}\" must be {} but is {}",
+            what, arg, INLINE_SYMBOL_WIDTH_1, width
+        )),
+    }
+}
+
+fn adapt_wrap_max_lines_argument(arg: String) -> usize {
+    if arg == "∞" || arg == "unlimited" || arg.starts_with("inf") {
+        0
+    } else {
+        arg.parse::<usize>()
+            .unwrap_or_else(|err| fatal(format!("Invalid wrap-max-lines argument: {}", err)))
+            + 1
+    }
+}
+
 #[derive(PartialEq)]
 enum Stop {
     StackEmpty,
@@ -449,13 +539,13 @@ pub fn wrap_minusplus_block<'c: 'a, 'a>(
         };
 
         if minus_extended > 0 {
-            new_states[Left].push(State::HunkMinus(None));
+            new_states[Left].push(State::HunkMinus(DiffType::Unified, None));
             for _ in 1..minus_extended {
                 new_states[Left].push(State::HunkMinusWrapped);
             }
         }
         if plus_extended > 0 {
-            new_states[Right].push(State::HunkPlus(None));
+            new_states[Right].push(State::HunkPlus(DiffType::Unified, None));
             for _ in 1..plus_extended {
                 new_states[Right].push(State::HunkPlusWrapped);
             }
@@ -473,7 +563,7 @@ pub fn wrap_minusplus_block<'c: 'a, 'a>(
 #[allow(clippy::comparison_chain, clippy::type_complexity)]
 pub fn wrap_zero_block<'c: 'a, 'a>(
     config: &'c Config,
-    raw_line: &str,
+    line: &str,
     mut states: Vec<State>,
     syntax_style_sections: Vec<LineSections<'a, SyntectStyle>>,
     diff_style_sections: Vec<LineSections<'a, Style>>,
@@ -500,7 +590,7 @@ pub fn wrap_zero_block<'c: 'a, 'a>(
     // If that changes the wrapping logic should be updated as well.
     debug_assert_eq!(diff_style_sections.len(), 1);
 
-    let should_wrap = line_is_too_long(raw_line, line_width);
+    let should_wrap = line_is_too_long(line, line_width);
 
     if should_wrap {
         let syntax_style = wrap_line(
@@ -550,11 +640,10 @@ mod tests {
 
     use super::wrap_line;
     use super::WrapConfig;
-    use crate::ansi::strip_ansi_codes;
     use crate::config::Config;
     use crate::paint::LineSections;
     use crate::style::Style;
-    use crate::tests::integration_test_utils::{make_config_from_args, run_delta};
+    use crate::tests::integration_test_utils::{make_config_from_args, DeltaTest};
 
     lazy_static! {
         static ref S1: Style = Style {
@@ -575,9 +664,9 @@ mod tests {
         static ref SD: Style = Style::default();
     }
 
-    const W: &str = &"+"; // wrap
-    const WR: &str = &"<"; // wrap-right
-    const RA: &str = &">"; // right-align
+    const W: &str = "+"; // wrap
+    const WR: &str = "<"; // wrap-right
+    const RA: &str = ">"; // right-align
 
     lazy_static! {
         static ref WRAP_DEFAULT_ARGS: Vec<&'static str> = vec![
@@ -606,7 +695,7 @@ mod tests {
     }
 
     fn mk_wrap_cfg(wrap_cfg: &WrapConfig) -> Config {
-        let mut cfg: Config = Config::from(make_config_from_args(&[]));
+        let mut cfg: Config = make_config_from_args(&[]);
         cfg.wrap_config = wrap_cfg.clone();
         cfg
     }
@@ -617,7 +706,7 @@ mod tests {
         <I as IntoIterator>::IntoIter: DoubleEndedIterator,
         S: Copy + Default + std::fmt::Debug,
     {
-        wrap_line(&cfg, line, line_width, &S::default(), &None)
+        wrap_line(cfg, line, line_width, &S::default(), &None)
     }
 
     #[test]
@@ -732,7 +821,7 @@ mod tests {
     }
 
     #[test]
-    fn test_wrap_line_newlines<'a>() {
+    fn test_wrap_line_newlines() {
         fn mk_input(len: usize) -> LineSections<'static, Style> {
             const IN: &str = "0123456789abcdefZ";
             let v = &[*S1, *S2];
@@ -785,10 +874,10 @@ mod tests {
             let line = mk_input_nl(9);
             let lines = wrap_test(&cfg, line, 3);
             let expected = mk_input_nl(9);
-            let line1 = mk_expected(&expected, 0, 2, Some((*SD, &W)));
-            let line2 = mk_expected(&expected, 2, 4, Some((*SD, &W)));
-            let line3 = mk_expected(&expected, 4, 6, Some((*SD, &W)));
-            let line4 = mk_expected(&expected, 6, 8, Some((*SD, &W)));
+            let line1 = mk_expected(&expected, 0, 2, Some((*SD, W)));
+            let line2 = mk_expected(&expected, 2, 4, Some((*SD, W)));
+            let line3 = mk_expected(&expected, 4, 6, Some((*SD, W)));
+            let line4 = mk_expected(&expected, 6, 8, Some((*SD, W)));
             let line5 = mk_expected(&expected, 8, 11, None);
             assert_eq!(lines, vec![line1, line2, line3, line4, line5]);
         }
@@ -797,10 +886,10 @@ mod tests {
             let line = mk_input_nl(10);
             let lines = wrap_test(&cfg, line, 3);
             let expected = mk_input_nl(10);
-            let line1 = mk_expected(&expected, 0, 2, Some((*SD, &W)));
-            let line2 = mk_expected(&expected, 2, 4, Some((*SD, &W)));
-            let line3 = mk_expected(&expected, 4, 6, Some((*SD, &W)));
-            let line4 = mk_expected(&expected, 6, 8, Some((*SD, &W)));
+            let line1 = mk_expected(&expected, 0, 2, Some((*SD, W)));
+            let line2 = mk_expected(&expected, 2, 4, Some((*SD, W)));
+            let line3 = mk_expected(&expected, 4, 6, Some((*SD, W)));
+            let line4 = mk_expected(&expected, 6, 8, Some((*SD, W)));
             let line5 = mk_expected(&expected, 8, 11, Some((*S2, "\n")));
             assert_eq!(lines, vec![line1, line2, line3, line4, line5]);
         }
@@ -844,8 +933,8 @@ mod tests {
         assert_eq!(
             lines,
             vec![
-                vec![(*S1, "abc"), (*SD, &W)],
-                vec![(*S2, "mnö̲"), (*SD, &W)],
+                vec![(*S1, "abc"), (*SD, W)],
+                vec![(*S2, "mnö̲"), (*SD, W)],
                 vec![(*S1, "xyz")]
             ]
         );
@@ -856,8 +945,8 @@ mod tests {
         assert_eq!(
             lines,
             vec![
-                vec![(*S1, "abc"), (*SD, &W)],
-                vec![(*S2, "deநி"), (*SD, &W)],
+                vec![(*S1, "abc"), (*SD, W)],
+                vec![(*S2, "deநி"), (*SD, W)],
                 vec![(*S1, "ghij")]
             ]
         );
@@ -903,7 +992,7 @@ index 223ca50..e69de29 100644
 
     #[test]
     fn test_wrap_with_unequal_hunk_zero_width() {
-        let mut config = make_config_from_args(&default_wrap_cfg_plus(&[
+        DeltaTest::with_args(&default_wrap_cfg_plus(&[
             "--side-by-side",
             "--line-numbers-left-format",
             "│L│",
@@ -913,26 +1002,23 @@ index 223ca50..e69de29 100644
             "40",
             "--line-fill-method",
             "spaces",
-        ]));
-        config.truncation_symbol = ">".into();
-
-        let output = run_delta(HUNK_ZERO_DIFF, &config);
-        let output = strip_ansi_codes(&output);
-        let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-        let expected = vec![
-            "│L│abcdefghijklm+   │RRRR│abcdefghijklm+",
-            "│L│nopqrstuvwxzy+   │RRRR│nopqrstuvwxzy+",
-            "│L│ 0123456789 0+   │RRRR│ 0123456789 0+",
-            "│L│123456789 012+   │RRRR│123456789 012+",
-            "│L│3456789 01234567>│RRRR│3456789 01234>",
-            "│L│a = 1            │RRRR│a = 2         ",
-        ];
-        assert_eq!(lines, expected);
+        ]))
+        .set_config(|cfg| cfg.truncation_symbol = ">".into())
+        .with_input(HUNK_ZERO_DIFF)
+        .expect_after_header(
+            r#"
+            │L│abcdefghijklm+   │RRRR│abcdefghijklm+
+            │L│nopqrstuvwxzy+   │RRRR│nopqrstuvwxzy+
+            │L│ 0123456789 0+   │RRRR│ 0123456789 0+
+            │L│123456789 012+   │RRRR│123456789 012+
+            │L│3456789 01234567>│RRRR│3456789 01234>
+            │L│a = 1            │RRRR│a = 2         "#,
+        );
     }
 
     #[test]
     fn test_wrap_with_large_hunk_zero_line_numbers() {
-        let mut config = make_config_from_args(&default_wrap_cfg_plus(&[
+        DeltaTest::with_args(&default_wrap_cfg_plus(&[
             "--side-by-side",
             "--line-numbers-left-format",
             "│LLL│",
@@ -942,53 +1028,49 @@ index 223ca50..e69de29 100644
             "60",
             "--line-fill-method",
             "ansi",
-        ]));
-        config.truncation_symbol = ">".into();
-
-        let output = run_delta(HUNK_ZERO_LARGE_LINENUMBERS_DIFF, &config);
-        let output = strip_ansi_codes(&output);
-        let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-        let expected = vec![
-            "│LLL│abcde+                   │WW   10   +- 101999 WW│abcde+",
-            "│LLL│fghij+                   │WW        +-        WW│fghij+",
-            "│LLL│klmno+                   │WW        +-        WW│klmno+",
-            "│LLL│pqrst+                   │WW        +-        WW│pqrst+",
-            "│LLL│uvwxzy 0123456789 012345>│WW        +-        WW│uvwxz>",
-            "│LLL│a = 1                    │WW        +- 102000 WW│a = 2",
-        ];
-        assert_eq!(lines, expected);
+        ]))
+        .set_config(|cfg| cfg.truncation_symbol = ">".into())
+        .with_input(HUNK_ZERO_LARGE_LINENUMBERS_DIFF)
+        .expect_after_header(
+            r#"
+            │LLL│abcde+                   │WW   10   +- 101999 WW│abcde+
+            │LLL│fghij+                   │WW        +-        WW│fghij+
+            │LLL│klmno+                   │WW        +-        WW│klmno+
+            │LLL│pqrst+                   │WW        +-        WW│pqrst+
+            │LLL│uvwxzy 0123456789 012345>│WW        +-        WW│uvwxz>
+            │LLL│a = 1                    │WW        +- 102000 WW│a = 2"#,
+        );
     }
 
     #[test]
     fn test_wrap_with_keep_markers() {
         use crate::features::side_by_side::ansifill::ODD_PAD_CHAR;
-        let mut config = make_config_from_args(&default_wrap_cfg_plus(&[
+        let t = DeltaTest::with_args(&default_wrap_cfg_plus(&[
             "--side-by-side",
             "--keep-plus-minus-markers",
             "--width",
             "45",
-        ]));
-        config.truncation_symbol = ">".into();
+        ]))
+        .set_config(|cfg| cfg.truncation_symbol = ">".into())
+        .with_input(HUNK_MP_DIFF)
+        .expect_after_header(
+            r#"
+            │  4 │ abcdefghijklmn+ │ 15 │ abcdefghijklmn+
+            │    │ opqrstuvwxzy 0+ │    │ opqrstuvwxzy 0+
+            │    │ 123456789 0123+ │    │ 123456789 0123+
+            │    │ 456789 0123456+ │    │ 456789 0123456+
+            │    │ 789 0123456789> │    │ 789 0123456789>
+            │  5 │-a = 0123456789+ │ 16 │+b = 0123456789+
+            │    │  0123456789 01+ │    │  0123456789 01+
+            │    │ 23456789 01234+ │    │ 23456789 01234+
+            │    │ 56789 01234567+ │    │ 56789 01234567+
+            │    │ 89              │    │ 89"#,
+            // this column here is^ where ODD_PAD_CHAR is inserted due to the odd 45 width
+        );
 
-        let output = run_delta(HUNK_MP_DIFF, &config);
-        let output = strip_ansi_codes(&output);
-        let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-        let expected = vec![
-            "│ 4  │ abcdefghijklmn+ │ 15 │ abcdefghijklmn+",
-            "│    │ opqrstuvwxzy 0+ │    │ opqrstuvwxzy 0+",
-            "│    │ 123456789 0123+ │    │ 123456789 0123+",
-            "│    │ 456789 0123456+ │    │ 456789 0123456+",
-            "│    │ 789 0123456789> │    │ 789 0123456789>",
-            "│ 5  │-a = 0123456789+ │ 16 │+b = 0123456789+",
-            "│    │  0123456789 01+ │    │  0123456789 01+",
-            "│    │ 23456789 01234+ │    │ 23456789 01234+",
-            "│    │ 56789 01234567+ │    │ 56789 01234567+",
-            "│    │ 89              │    │ 89",
-            // this is place where ^ ODD_PAD_CHAR is inserted due to the odd 45 width
-        ];
-        assert_eq!(lines, expected);
+        assert!(!t.output.is_empty());
 
-        for line in lines {
+        for line in t.output.lines().skip(crate::config::HEADER_LEN) {
             assert_eq!(line.chars().nth(22), Some(ODD_PAD_CHAR));
         }
     }
@@ -997,41 +1079,34 @@ index 223ca50..e69de29 100644
     fn test_alignment_2_lines_vs_3_lines() {
         let config =
             make_config_from_args(&default_wrap_cfg_plus(&["--side-by-side", "--width", "55"]));
+
         {
-            let output = run_delta(
-                &format!(
+            DeltaTest::with_config(&config)
+                .with_input(&format!(
                     "{}-{}+{}",
                     HUNK_ALIGN_DIFF_HEADER, HUNK_ALIGN_DIFF_SHORT, HUNK_ALIGN_DIFF_LONG
-                ),
-                &config,
-            );
-            let output = strip_ansi_codes(&output);
-            let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-            let expected = vec![
-                "│ 1  │.........1.........2< │ 1  │.........1.........2+",
-                "│    │                >.... │    │.........3.........4+",
-                "│    │                      │    │.........5.........6",
-                // place where ODD_PAD_CHAR ^ is inserted due to the odd 55 width
-            ];
-            assert_eq!(lines, expected);
+                ))
+                .expect_after_header(
+                    r#"
+                    │  1 │.........1.........2< │  1 │.........1.........2+
+                    │    │                >.... │    │.........3.........4+
+                    │    │                      │    │.........5.........6"#,
+                );
+            // the place where ODD_PAD_CHAR^ is inserted due to the odd 55 width
         }
 
         {
-            let output = run_delta(
-                &format!(
+            DeltaTest::with_config(&config)
+                .with_input(&format!(
                     "{}-{}+{}",
                     HUNK_ALIGN_DIFF_HEADER, HUNK_ALIGN_DIFF_LONG, HUNK_ALIGN_DIFF_SHORT
-                ),
-                &config,
-            );
-            let output = strip_ansi_codes(&output);
-            let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-            let expected = vec![
-                "│ 1  │.........1.........2+ │ 1  │.........1.........2<",
-                "│    │.........3.........4+ │    │                >....",
-                "│    │.........5.........6  │    │",
-            ];
-            assert_eq!(lines, expected);
+                ))
+                .expect_after_header(
+                    r#"
+                    │  1 │.........1.........2+ │  1 │.........1.........2<
+                    │    │.........3.........4+ │    │                >....
+                    │    │.........5.........6  │    │"#,
+                );
         }
     }
 
@@ -1046,39 +1121,31 @@ index 223ca50..e69de29 100644
         ]));
 
         {
-            let output = run_delta(
-                &format!(
+            DeltaTest::with_config(&config)
+                .with_input(&format!(
                     "{}-{}+{}",
                     HUNK_ALIGN_DIFF_HEADER, HUNK_ALIGN_DIFF_SHORT, HUNK_ALIGN_DIFF_LONG
-                ),
-                &config,
-            );
-            let output = strip_ansi_codes(&output);
-            let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-            let expected = vec![
-                "│ 1  │.........1.........2....│ 1  │.........1.........2...+",
-                "│    │                        │    │......3.........4......+",
-                "│    │                        │    │...5.........6          ",
-            ];
-            assert_eq!(lines, expected);
+                ))
+                .expect_after_header(
+                    r#"
+                    │  1 │.........1.........2....│  1 │.........1.........2...+
+                    │    │                        │    │......3.........4......+
+                    │    │                        │    │...5.........6          "#,
+                );
         }
 
         {
-            let output = run_delta(
-                &format!(
+            DeltaTest::with_config(&config)
+                .with_input(&format!(
                     "{}-{}+{}",
                     HUNK_ALIGN_DIFF_HEADER, HUNK_ALIGN_DIFF_LONG, HUNK_ALIGN_DIFF_SHORT
-                ),
-                &config,
-            );
-            let output = strip_ansi_codes(&output);
-            let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-            let expected = vec![
-                "│ 1  │.........1.........2...+│ 1  │.........1.........2....",
-                "│    │......3.........4......+│    │",
-                "│    │...5.........6          │    │",
-            ];
-            assert_eq!(lines, expected);
+                ))
+                .expect_after_header(
+                    r#"
+                    │  1 │.........1.........2...+│  1 │.........1.........2....
+                    │    │......3.........4......+│    │
+                    │    │...5.........6          │    │"#,
+                );
         }
     }
 
@@ -1097,39 +1164,31 @@ index 223ca50..e69de29 100644
         config.truncation_symbol = ">".into();
 
         {
-            let output = run_delta(
-                &format!(
+            DeltaTest::with_config(&config)
+                .with_input(&format!(
                     "{}-{}+{}",
                     HUNK_ALIGN_DIFF_HEADER, HUNK_ALIGN_DIFF_SHORT, HUNK_ALIGN_DIFF_LONG
-                ),
-                &config,
-            );
-            let output = strip_ansi_codes(&output);
-            let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-            let expected = vec![
-                "│ 1  │.........1.........2....      │ 1  │.........1.........2.........+",
-                "│    │                              │    │3.........4.........5........+",
-                "│    │                              │    │.6                            ",
-            ];
-            assert_eq!(lines, expected);
+                ))
+                .expect_after_header(
+                    r#"
+                    │  1 │.........1.........2....      │  1 │.........1.........2.........+
+                    │    │                              │    │3.........4.........5........+
+                    │    │                              │    │.6                            "#,
+                );
         }
 
         {
             config.wrap_config.max_lines = 2;
-            let output = run_delta(
-                &format!(
+            DeltaTest::with_config(&config)
+                .with_input(&format!(
                     "{}-{}+{}",
                     HUNK_ALIGN_DIFF_HEADER, HUNK_ALIGN_DIFF_SHORT, HUNK_ALIGN_DIFF_LONG
-                ),
-                &config,
-            );
-            let output = strip_ansi_codes(&output);
-            let lines: Vec<_> = output.lines().skip(crate::config::HEADER_LEN).collect();
-            let expected = vec![
-                "│ 1  │.........1.........2....      │ 1  │.........1.........2.........+",
-                "│    │                              │    │3.........4.........5........>",
-            ];
-            assert_eq!(lines, expected);
+                ))
+                .expect_after_header(
+                    r#"
+                    │  1 │.........1.........2....      │  1 │.........1.........2.........+
+                    │    │                              │    │3.........4.........5........>"#,
+                );
         }
     }
 }
